# Custom CanvasLayer to pull out color from CanvasImage
class CanvasLayer extends Layer
	
	constructor: (options) ->
		super options
	
		@_canvas = document.createElement("canvas")
		
		@_canvas.setAttribute("width", @width)
		@_canvas.setAttribute("height", @height)
		
		@_element.appendChild(@_canvas)
		
		@_ctx = @_canvas.getContext("2d")

		@_ctx.webkitImageSmoothingEnabled = true

	#Draws an Image on Canvas		
	@define "canvasImage",
		get: -> @_canvasImage
		set: (value) ->
			
			@_canvasImage = value
			
			@_image = new Image
			@_image.src = value
			
			@_image.onload = =>
			
				@_canvasLoadedImage = @_image
				@_ctx.drawImage(@_canvasLoadedImage, 0, 0, @width, @height)
				#Takes care of color update function
				updatecolor(currentindex)

	#Getting from actual sourceImage, if not you'll get typeerror
	@define "sourceImage",
		get: -> @_canvasLoadedImage
					
#Audiolayer is simple tweak of videolayer by Koen Bok.
class AudioLayer extends Layer

	constructor: (options={}) ->

		super options

		@player = document.createElement("audio")
		@player.style.width = "100%"
		@player.style.height = "100%"

		# Make it work with .on and .off
		# https://developer.mozilla.org/en-US/docs/Web/Guide/Events/Media_events
		@player.on = @player.addEventListener
		@player.off = @player.removeEventListener

		@audio = options.audio

		@_element.appendChild(@player)

	@define "audio",
		get: -> @audio.src
		set: (audio) -> @player.src = audio
		
#Define Colorthief, It's that simple
colorThief = new ColorThief()

#Setting Up Default state.
currentindex = 1

bg = new BackgroundLayer 

# Image / Text Array
albumArts = ["images/albumArt1.jpg","images/albumArt2.jpg","images/albumArt3.jpg","images/albumArt4.jpg","images/albumArt5.jpg"]
songtitle = ["<h2>SuperModel</h2><br><p>Foster The People</p>","<h2>Halcyon Days</h2><br><p>Ellie Goulding</p>","<h2>In a Perfect World</h2><br><p>Kodaline</p>","<h2>No Fear</h2><br><p>Common</p>","<h2>Lights and Camera</h2><br><p>Yuna</p>"]

# Basic Stuffs, Crappy Naming Though
container = new Layer x:0, y:0, width:360, height:640, backgroundColor:'transparent',shadowY:4,shadowBlur:15,shadowColor:'rgba(0,0,0,0.5)'
thisis = new Layer x:0, y:0, width:360, height:640, backgroundColor:''
player = new CanvasLayer 
	x:0, y:0, width:360, height:360
	
#Setting Up The Image for Canvas
player.canvasImage = albumArts[currentindex]

#Layer Hierarchy
bg.addSubLayer(container)
container.center()
container.addSubLayer(thisis)
container.addSubLayer(player)

#Music Control
musiccontrol = new AudioLayer width:0, height:0, audio:"audio/music.mp3"
#Music Playing Status
isplaying = false

#Control Components
progressbg = new Layer x:0, y:520, width:360, height:4, backgroundColor:'rgba(0,0,0,0.2)'
progressbar = new Layer x:0, y:520, width:0, height:4
metadata = new Layer x:0, y:player.y+player.height, width:360, 
height:160
metadata.html = songtitle[currentindex]
metadata.style = 
	fontFamily:'Roboto' #Google Web Font
	fontSize:'16px'
	color:'#fff'
	padding:'30px 0px 0px 15px'
	lineHeight:'0.8'
	fontWeight:'400'
	
playlist = new Layer x:0, y:progressbar.y+progressbar.height, width:360, height:100, backgroundColor:'#fff'
playlist.html = "Up Next"
playlist.style = 
	fontFamily:'Roboto' #Google Web Font
	fontSize:'13px'
	color:'#aaa'
	padding:'20px 0px 0px 15px'
	lineHeight:'0.8'
	fontWeight:'400'

	
fabbtn = new Layer x:0, y:0, width:65,height:65, borderRadius:'40px', backgroundColor:'#f0f0f0',shadowY:4,shadowBlur:15,shadowColor:'rgba(0,0,0,0.5)'
btnshuffle = new Layer x:0, y:0, width:20, height:21, image:'images/btnshuffle.png'

fabbtn.addSubLayer(btnshuffle)
btnshuffle.centerX()
btnshuffle.centerY()

#Controller Layer
controller = new Layer x:0, width:360, height:54, backgroundColor:'rgba(0,0,0,0.3)'
progressnob = new Layer x:0, y:progressbg.y, width:30,height:30, backgroundColor:'transparent'

#Buttons
icnpause = new Layer 
	x:0, y:0, width:48, height:48, image:"images/icnpause.png"
icnplay = new Layer 
	x:0, y:0, width:48, height:48, image:"images/icnplay.png"	
control = new Layer 
	x:0, y:0, width:48, height:48, backgroundColor:'transparent'

btnnext = new Layer 
	x:0, y:0, width:48, height:48, image:"images/icnnext.png"
btnprev = new Layer 
	x:0, y:0, width:48, height:48, image:"images/icnprev.png"	
icnpause.opacity = 0
icnpause.scale = 0

container.addSubLayer(progressnob)
container.addSubLayer(progressbg)
container.addSubLayer(progressbar)
container.addSubLayer(metadata)
container.addSubLayer(playlist)
controller.addSubLayer(control)
controller.addSubLayer(icnpause)
controller.addSubLayer(icnplay)
controller.addSubLayer(btnnext)
controller.addSubLayer(btnprev)
container.addSubLayer(controller)

btnprev.centerY()
btnnext.centerY()
control.center()
icnpause.center()
icnplay.center()
controller.y = metadata.y+metadata.height-54
btnnext.x = 260
btnprev.x = 51

#This is the Key
updatecolor = () ->
			
	player.opacity = 0.5
	
	player.animate
		properties:
			opacity:1
		curve:'spring(100,12,0)'

	#Getting Palette from Image				
	palette = colorThief.getPalette(player.sourceImage,20) #It's sourceImage, not canvas
	colorone = palette[0] #Getting values from Array, It's Random Numbers that I picked.
	colortwo = palette[12]
	colorthr = palette[3]
	colorfor = palette[12]
	colorfiv = palette[2]
	
	# Changing Font Color based on backgroundColor
	if colorthr[0]+colorthr[1]+colorthr[2] > 500 
		metadata.style = color:'#333'
	else 
		metadata.style = color:'#fff'

	#Animate Transition				
	metadata.opacity = 0.5
		
	metadata.animate
		properties:
			opacity:1
		curve:'ease'
		time:0.6

	#Setting Up The Color.
	bg.backgroundColor = 'rgb('+colorfiv[0]+','+colorfiv[1]+','+colorfiv[2]+')'
	thisis.backgroundColor = 'rgb('+colorone[0]+','+colorone[1]+','+colorone[2]+')'
	progressbar.backgroundColor = 'rgb('+colortwo[0]+','+colortwo[1]+','+colortwo[2]+')'
	metadata.backgroundColor = 'rgb('+colorthr[0]+','+colorthr[1]+','+colorthr[2]+')'
	fabbtn.backgroundColor = 'rgb('+colorfor[0]+','+colorfor[1]+','+colorfor[2]+')'
	

musiccontrol.player.on "canplay", ->
	
	#Do Something when audioplayer is ready to play	
musiccontrol.player.on "timeupdate", ->
	
	#Showing Progress
	
	progress = musiccontrol.player.currentTime/musiccontrol.player.duration
	progressnob.x = progress*640
	progressbar.width = progressnob.x

progressnob.draggable.enabled = true
progressnob.draggable.speedY = 0

progressnob.on Events.DragStart, ->
	
		musiccontrol.player.pause()
		musiccontrol.player.load()
	
		progressbar.width = Math.round(progressnob.x+progressnob.width)

		
myprogress = 0

progressnob.on Events.DragMove, ->
				
		myprogress = progressnob.x
		progressbar.width = Math.round(myprogress+progressnob.width)

		
		progressbar.width = Math.round(myprogress.x+progressnob.width)
		
progressnob.on Events.DragEnd, ->

			
			newpositon = progressnob.x/640*musiccontrol.player.duration
			
			musiccontrol.player.currentTime = newpositon
			musiccontrol.player.play()
						
			progressbar.animate
				properties:
					width:Math.round(progressnob.x+progressnob.width)
				curve:'spring(100,12,0)'

				icnpause.animate
					properties:
						scale:1
						opacity:1
					curve:'spring(200,12,0)'

				icnplay.animate
					properties:
						scale:0
						opacity:0
					curve:'spring(100,12,0)'

	
musiccontrol.player.on "seeking", ->
	
	#Getting Event from Seeking Event

btnprev.on Events.Click, ->
	
	#Prev/Next Song, but only with color Transition	
	btnprev.scale = 0.8

	btnprev.animate
		properties:
			scale:1
		curve:'spring(100,12,0)'
	
	if currentindex > 0
		currentindex = currentindex-1
	else 
		currentindex = 0
	
	player.canvasImage = albumArts[currentindex]
	metadata.html = songtitle[currentindex]

	progressnob.animate
		properties:	
			x:0
		curve:'spring(100,12,0)'
		
	progressbar.animate
		properties:
			width:0		
		curve:'spring(100,12,0)'
	
btnnext.on Events.Click, ->

	#Prev/Next Song, but only with color Transition	
	btnnext.scale = 0.8
	
	btnnext.animate
		properties:
			scale:1
		curve:'spring(100,12,0)'

	if currentindex >= 4
		currentindex = 4
	else
		currentindex++

	player.canvasImage = albumArts[currentindex]
	metadata.html = songtitle[currentindex]

control.on Events.TouchStart, ->
	
	if isplaying == true
		musiccontrol.player.pause()
		
		icnpause.animate
			properties:
				scale:0
				opacity:0
			curve:'spring(200,12,0)'
		
		icnplay.animate
			properties:
				scale:1
				opacity:1
			curve:'spring(100,12,0)'
		
		isplaying = false
	
	else
		musiccontrol.player.play()
		
		icnpause.animate
			properties:
				scale:1
				opacity:1
			curve:'spring(100,12,0)'
		
		icnplay.animate
			properties:
				scale:0
				opacity:0
			curve:'spring(200,12,0)'
			
		isplaying = true
statusbar = new Layer 
	x:0, y:0, width:360, height:23, image:"images/statusbar.png"
navand = new Layer 
	x:0, y:0, width:360, height:48, image:"images/navbar-and.png"
	
touchripple = new Layer x:0, y:0, width:65, height:65, backgroundColor:'rgba(255,255,255,0.5)', borderRadius:'40px',opacity:0
fabbtn.addSubLayer(touchripple)
fabbtn.clip = yes

#Randomly show

fabbtn.on Events.Click, ->

	shufflenumber = Math.floor(Math.random()*(1+4 - 1))+1
	currentindex = shufflenumber
	player.canvasImage = albumArts[shufflenumber]
	metadata.html = songtitle[shufflenumber]
	
	touchripple.opacity = 1
	touchripple.scale = 0
	
	touchripple.animate
		properties:
			scale:1
			opacity:0
		curve:'ease'
		time:0.2

	btnshuffle.rotation = 180
	
	btnshuffle.animate
		properties:
			rotation:0
		curve:'spring(100,12,0)'
		
	fabbtn.shadowY = 2
	fabbtn.shadowBlur = 8
	fabbtn.shadowColor = 'rgba(0,0,0,0.3)'
	fabbtn.scale = 0.95
		
	Utils.delay 0.3, -> fabbtn.shadowColor = 'rgba(0,0,0,0.5)'
	
	fabbtn.animate
		properties:
			scale:1
			shadowY:4
			shadowBlur:15
		curve:'spring(100,12,0)'

actionbar = new Layer 
	x:15, y:40, width:161, height:20, image:"images/nowplaying.png"		
container.addSubLayer(statusbar)
container.addSubLayer(navand)
container.addSubLayer(actionbar)
container.addSubLayer(fabbtn)
fabbtn.y = 334
fabbtn.x = 288
navand.y = 592
